@@ -1,6 +1,6 @@
 // Filename: lib.rs
-// Version:	 0.12
-// Date:	 19-11-2021 (DD-MM-YYYY)
+// Version:	 0.14
+// Date:	 30-12-2021 (DD-MM-YYYY)
 // Library:  gpcas_cpu_model
 //
 // Copyright (c) 2021 Kai Rese
@@ -25,49 +25,51 @@
 //! to ensure compatibility.
 #![warn(missing_docs)]
 
-pub mod component_config;
+pub mod config;
 mod legacy;
+mod validation;
 
 use gpcas_base::file::{deserialize_upgrade_from, DeserializeFunction, GpcasFileStruct};
 use gpcas_base::instruction_type;
+use gpcas_isa::Isa;
 use serde::{Deserialize, Serialize};
+use std::iter::once;
+
+pub use validation::{validate, FailedValidation, ValidationError};
 
 /// The configuration of an abstract CPU model.
 #[derive(Clone, Default, Deserialize, Serialize)]
-pub struct CPUModel {
+pub struct CpuModel {
     /// The maximum supported vector size of the model in bits.
     pub max_vector_size: usize,
     /// The ALU latency of each instruction class.
     pub execution_latencies: InstructionLatencies,
     /// Configuration for the fetch stage of the front end.
-    pub fetch_config: component_config::FetchConfig,
+    pub fetch: config::Fetch,
     /// Configuration for the memory controller.
-    pub memory_controller_config: component_config::MemoryControllerConfig,
+    pub memory_controller: config::MemoryController,
     /// Caches of the memory hierarchy.
-    pub caches: Vec<component_config::CacheConfig>,
+    pub caches: Vec<config::Cache>,
     /// Configuration for the decode stage of the front end.
-    pub decode_config: component_config::DecodeConfig,
+    pub decoder: config::Decoder,
     /// Configuration for the reorder buffer.
-    pub reorder_buffer_config: component_config::ReorderBufferConfig,
+    pub reorder_buffer: config::ReorderBuffer,
     /// How many cycles the dispatch unit needs to dispatch an instruction.
     pub dispatch_cycle_count: usize,
     /// Configuration of the register file.
-    pub reg_file_config: component_config::RegisterFileConfig,
+    pub register_file: config::RegisterFile,
     /// Configuration of schedulers.
-    pub schedulers: Vec<component_config::SchedulerConfig>,
+    pub schedulers: Vec<config::Scheduler>,
     /// Configuration for each pipeline from just after the front end to the last stage.
-    pub pipelines: Vec<component_config::PipelineConfig>,
-    /// Configuration of the load-store-unit.
-    pub load_store_unit_config: component_config::LoadStoreUnitConfig,
+    pub pipelines: Vec<config::Pipeline>,
+    /// Configuration of the load/store unit.
+    pub load_store_unit: config::LoadStoreUnit,
 }
 
-/// Defines the amount of clock cycles each instruction type needs.
+/// Defines the amount of clock cycles each instruction type needs to execute.
 ///
 /// The fields of this struct are used to selectively overwrite default values. If a field is
 /// `None`, the default value is used.
-///
-/// A value of zero means that the type isn't supported in a pipeline. By default, all types are
-/// supported.
 #[derive(Clone, Default, Deserialize, Serialize)]
 pub struct InstructionLatencies {
     /// Typically takes multiple, but few clock cycles.
@@ -88,24 +90,32 @@ pub struct InstructionLatencies {
     pub float_multiply_add: Option<u16>,
 }
 
-impl CPUModel {
+impl CpuModel {
     /// Creates a new model without components and default global values.
     pub fn new() -> Self {
-        CPUModel {
+        CpuModel {
             max_vector_size: 128,
             execution_latencies: InstructionLatencies::default(),
-            fetch_config: component_config::FetchConfig::default(),
-            memory_controller_config: component_config::MemoryControllerConfig::default(),
+            fetch: config::Fetch::default(),
+            memory_controller: config::MemoryController::default(),
             caches: Vec::new(),
-            decode_config: component_config::DecodeConfig::default(),
-            reorder_buffer_config: Default::default(),
+            decoder: config::Decoder::default(),
+            reorder_buffer: Default::default(),
             dispatch_cycle_count: 0,
-            reg_file_config: Default::default(),
+            register_file: Default::default(),
             schedulers: Vec::new(),
             pipelines: Vec::new(),
-            load_store_unit_config: Default::default(),
+            load_store_unit: Default::default(),
         }
     }
+
+    /// Validates the model for a correct configuration.
+    ///
+    /// This does the same as [validation::validate] and can be used if one prefers an
+    /// object oriented interface.
+    pub fn validate(&self, isa: &Isa) -> Result<(), FailedValidation> {
+        validation::validate(self, isa)
+    }
 }
 
 impl InstructionLatencies {
@@ -135,15 +145,26 @@ impl InstructionLatencies {
             1,
         ]
     }
+
+    /// Return an iterator over every latency defined in the struct.
+    pub fn iter(&self) -> impl std::iter::Iterator<Item = Option<u16>> {
+        once(self.integer_multiply)
+            .chain(once(self.integer_divide))
+            .chain(once(self.integer_multiply_add))
+            .chain(once(self.integer_shift))
+            .chain(once(self.float_add))
+            .chain(once(self.float_multiply))
+            .chain(once(self.integer_divide))
+            .chain(once(self.float_multiply_add))
+    }
 }
 
-impl GpcasFileStruct for CPUModel {
+impl GpcasFileStruct for CpuModel {
     const FILE_IDENTIFIER: &'static str = "gpcas::cpu_model";
-    const CURRENT_FILE_VERSION: usize = 3;
+    const CURRENT_FILE_VERSION: usize = 4;
     const COMPATIBLE_VERSIONS: &'static [(usize, DeserializeFunction<Self>)] = &[
-        (0, deserialize_upgrade_from::<legacy::v0::CPUModel, Self>),
-        (1, deserialize_upgrade_from::<legacy::v1::CPUModel, Self>),
         (2, deserialize_upgrade_from::<legacy::v2::CPUModel, Self>),
+        (3, deserialize_upgrade_from::<legacy::v3::CPUModel, Self>),
     ];
 }
 
